@@ -161,6 +161,14 @@ SUPPORT_TRANSFER_PAGINATION = '3.59'
 
 VOLUME_TIME_COMPARISON_FILTER = '3.60'
 
+VOLUME_CLUSTER_NAME = '3.61'
+
+DEFAULT_TYPE_OVERRIDES = '3.62'
+
+VOLUME_TYPE_ID_IN_VOLUME_DETAIL = '3.63'
+
+ENCRYPTION_KEY_ID_IN_DETAILS = '3.64'
+
 
 def get_mv_header(version):
     """Gets a formatted HTTP microversion header.
