@@ -12,11 +12,11 @@
 #    under the License.
 
 """The volumes V3 api."""
+from http import HTTPStatus
 
 from oslo_log import log as logging
 from oslo_log import versionutils
-import six
-from six.moves import http_client
+from oslo_utils import timeutils
 import webob
 from webob import exc
 
@@ -78,7 +78,7 @@ class VolumeController(volumes_v2.VolumeController):
                                cascade=cascade,
                                force=force)
 
-        return webob.Response(status_int=http_client.ACCEPTED)
+        return webob.Response(status_int=HTTPStatus.ACCEPTED)
 
     @common.process_general_filtering('volume')
     def _process_volume_filtering(self, context=None, filters=None,
@@ -89,10 +89,44 @@ class VolumeController(volumes_v2.VolumeController):
         if req_version.matches(None, mv.BACKUP_UPDATE):
             filters.pop('group_id', None)
 
+        if req_version.matches(None, mv.SUPPORT_TRANSFER_PAGINATION):
+            filters.pop('created_at', None)
+            filters.pop('updated_at', None)
+
         api_utils.remove_invalid_filter_options(
             context, filters,
             self._get_volume_filter_options())
 
+    def _handle_time_comparison_filters(self, filters):
+        for time_comparison_filter in ['created_at', 'updated_at']:
+            if time_comparison_filter in filters:
+                time_filter_dict = {}
+                comparison_units = filters[time_comparison_filter].split(',')
+                operators = common.get_time_comparsion_operators()
+                for comparison_unit in comparison_units:
+                    try:
+                        operator_and_time = comparison_unit.split(":")
+                        comparison_operator = operator_and_time[0]
+                        time = ''
+                        for time_str in operator_and_time[1:-1]:
+                            time += time_str + ":"
+                        time += operator_and_time[-1]
+                        if comparison_operator not in operators:
+                            msg = _(
+                                'Invalid %s operator') % comparison_operator
+                            raise exc.HTTPBadRequest(explanation=msg)
+                    except IndexError:
+                        msg = _('Invalid %s value') % time_comparison_filter
+                        raise exc.HTTPBadRequest(explanation=msg)
+                    try:
+                        parsed_time = timeutils.parse_isotime(time)
+                    except ValueError:
+                        msg = _('Invalid %s value') % time
+                        raise exc.HTTPBadRequest(explanation=msg)
+                    time_filter_dict[comparison_operator] = parsed_time
+
+                filters[time_comparison_filter] = time_filter_dict
+
     def _get_volumes(self, req, is_detail):
         """Returns a list of volumes, transformed through view builder."""
 
@@ -120,6 +154,8 @@ class VolumeController(volumes_v2.VolumeController):
         if 'name' in filters:
             filters['display_name'] = filters.pop('name')
 
+        self._handle_time_comparison_filters(filters)
+
         strict = req.api_version_request.matches(
             mv.VOLUME_LIST_BOOTABLE, None)
         self.volume_api.check_volume_filters(filters, strict)
@@ -172,7 +208,7 @@ class VolumeController(volumes_v2.VolumeController):
         return view_builder_v3.quick_summary(num_vols, int(sum_size),
                                              all_distinct_metadata)
 
-    @wsgi.response(http_client.ACCEPTED)
+    @wsgi.response(HTTPStatus.ACCEPTED)
     @wsgi.Controller.api_version(mv.VOLUME_REVERT)
     @wsgi.action('revert')
     def revert(self, req, id, body):
@@ -205,9 +241,7 @@ class VolumeController(volumes_v2.VolumeController):
                            's_id': l_snap.id})
             self.volume_api.revert_to_snapshot(context, volume, l_snap)
         except (exception.InvalidVolume, exception.InvalidSnapshot) as e:
-            raise exc.HTTPConflict(explanation=six.text_type(e))
-        except exception.VolumeSizeExceedsAvailableQuota as e:
-            raise exc.HTTPForbidden(explanation=six.text_type(e))
+            raise exc.HTTPConflict(explanation=str(e))
 
     def _get_image_snapshot(self, context, image_uuid):
         image_snapshot = None
@@ -234,7 +268,7 @@ class VolumeController(volumes_v2.VolumeController):
                             raise exc.HTTPNotFound(explanation=explanation)
             return image_snapshot
 
-    @wsgi.response(http_client.ACCEPTED)
+    @wsgi.response(HTTPStatus.ACCEPTED)
     @validation.schema(volumes.create, mv.BASE_VERSION,
                        mv.get_prior_version(mv.GROUP_VOLUME))
     @validation.schema(volumes.create_volume_v313, mv.GROUP_VOLUME,
