@@ -14,17 +14,16 @@
 #
 
 import datetime
-import fixtures
+import functools
 import socket
-import sys
+from unittest import mock
 import uuid
 
-import mock
+import fixtures
 from oslo_config import cfg
 from oslo_service import loopingcall
 from oslo_utils import timeutils
 import oslo_versionedobjects
-import six
 
 from cinder.common import constants
 from cinder import context
@@ -109,6 +108,12 @@ def create_volume(ctxt,
 def attach_volume(ctxt, volume_id, instance_uuid, attached_host,
                   mountpoint, mode='rw'):
 
+    if isinstance(volume_id, objects.Volume):
+        volume_ovo = volume_id
+        volume_id = volume_ovo.id
+    else:
+        volume_ovo = None
+
     now = timeutils.utcnow()
     values = {}
     values['volume_id'] = volume_id
@@ -120,6 +125,13 @@ def attach_volume(ctxt, volume_id, instance_uuid, attached_host,
     volume, updated_values = db.volume_attached(
         ctxt, attachment['id'], instance_uuid,
         attached_host, mountpoint, mode)
+
+    if volume_ovo:
+        cls = objects.Volume
+        expected_attrs = cls._get_expected_attrs(ctxt)
+        volume = cls._from_db_object(ctxt, cls(ctxt), volume,
+                                     expected_attrs=expected_attrs)
+
     return volume
 
 
@@ -443,7 +455,7 @@ file_spec = None
 
 
 def get_file_spec():
-    """Return a Python 2 and 3 compatible version of a 'file' spec.
+    """Return a 'file' spec.
 
     This is to be used anywhere that you need to do something such as
     mock.MagicMock(spec=file) to mock out something with the file attributes.
@@ -454,14 +466,9 @@ def get_file_spec():
     global file_spec
     # set on first use
     if file_spec is None:
-        if sys.version_info[0] == 3:
-            import _io
-            file_spec = list(set(dir(_io.TextIOWrapper)).union(
-                set(dir(_io.BytesIO))))
-        else:
-            # NOTE(jsbryant): Pep8 on py3 based systems will fail because
-            # 'file' has been removed.  Using noqa here to avoid the failure.
-            file_spec = file  # noqa
+        import _io
+        file_spec = list(set(dir(_io.TextIOWrapper)).union(
+            set(dir(_io.BytesIO))))
 
 
 def generate_timeout_series(timeout):
@@ -550,7 +557,7 @@ def set_timeout(timeout):
 
     def _decorator(f):
 
-        @six.wraps(f)
+        @functools.wraps(f)
         def _wrapper(self, *args, **kwargs):
             self.useFixture(fixtures.Timeout(timeout, gentle=True))
             return f(self, *args, **kwargs)
