@@ -1,4 +1,4 @@
-# Copyright (c) 2017-2019 Dell Inc. or its subsidiaries.
+# Copyright (c) 2020 Dell Inc. or its subsidiaries.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -17,7 +17,7 @@ from copy import deepcopy
 from unittest import mock
 
 from cinder import exception
-from cinder import test
+from cinder.tests.unit import test
 from cinder.tests.unit.volume.drivers.dell_emc.powermax import (
     powermax_data as tpd)
 from cinder.tests.unit.volume.drivers.dell_emc.powermax import (
@@ -38,9 +38,9 @@ class PowerMaxMaskingTest(test.TestCase):
         self.replication_device = self.data.sync_rep_device
         configuration = tpfo.FakeConfiguration(
             None, 'MaskingTests', 1, 1, san_ip='1.1.1.1',
-            san_login='smc', vmax_array=self.data.array, vmax_srp='SRP_1',
-            san_password='smc', san_api_port=8443,
-            vmax_port_groups=[self.data.port_group_name_f],
+            san_login='smc', powermax_array=self.data.array,
+            powermax_srp='SRP_1', san_password='smc', san_api_port=8443,
+            powermax_port_groups=[self.data.port_group_name_f],
             replication_device=self.replication_device)
         self._gather_info = common.PowerMaxCommon._gather_info
         common.PowerMaxCommon._get_u4p_failover_info = mock.Mock()
@@ -80,9 +80,11 @@ class PowerMaxMaskingTest(test.TestCase):
             self.driver.masking._sanity_port_group_check,
             None, self.data.array)
 
+    @mock.patch.object(
+        masking.PowerMaxMasking, '_check_director_and_port_status')
     @mock.patch.object(rest.PowerMaxRest, 'get_portgroup',
                        return_value=tpd.PowerMaxData.portgroup)
-    def test_sanity_port_group_check(self, mock_pg):
+    def test_sanity_port_group_check(self, mock_pg, mock_check):
         self.driver.masking._sanity_port_group_check(
             self.data.port_group_name_f, self.data.array)
 
@@ -196,6 +198,8 @@ class PowerMaxMaskingTest(test.TestCase):
             self.data.storagegroup_name_i, self.data.storagegroup_name_f,
             self.data.extra_specs, True)
 
+    @mock.patch.object(
+        masking.PowerMaxMasking, '_check_director_and_port_status')
     @mock.patch.object(rest.PowerMaxRest, 'get_masking_view',
                        side_effect=[tpd.PowerMaxData.maskingview,
                                     tpd.PowerMaxData.maskingview, None])
@@ -206,7 +210,7 @@ class PowerMaxMaskingTest(test.TestCase):
     @mock.patch.object(masking.PowerMaxMasking, '_create_new_masking_view',
                        return_value=None)
     def test_get_or_create_masking_view(self, mock_create_mv, mock_validate_mv,
-                                        mock_get_mv):
+                                        mock_get_mv, mock_check):
         for x in range(0, 3):
             self.driver.masking._get_or_create_masking_view(
                 self.data.array, self.maskingviewdict,
@@ -358,11 +362,13 @@ class PowerMaxMaskingTest(test.TestCase):
         self.assertEqual(7, mock_get_sg.call_count)
         self.assertEqual(1, mock_move.call_count)
 
+    @mock.patch.object(
+        masking.PowerMaxMasking, '_check_director_and_port_status')
     @mock.patch.object(
         rest.PowerMaxRest, 'get_portgroup',
         side_effect=([tpd.PowerMaxData.port_group_name_i, None]))
     def test_check_port_group(
-            self, mock_get_pg):
+            self, mock_get_pg, mock_check):
         for x in range(0, 2):
             _, msg = self.driver.masking._check_port_group(
                 self.data.array, self.maskingviewdict['maskingview_name'])
@@ -394,13 +400,13 @@ class PowerMaxMaskingTest(test.TestCase):
     def test_check_existing_initiator_group(self):
         with mock.patch.object(
                 rest.PowerMaxRest, 'get_element_from_masking_view',
-                return_value=tpd.PowerMaxData.inititiatorgroup):
+                return_value=tpd.PowerMaxData.initiatorgroup_name_f):
             ig_from_mv, msg = (
                 self.driver.masking._check_existing_initiator_group(
                     self.data.array, self.maskingviewdict['maskingview_name'],
                     self.maskingviewdict, self.data.storagegroup_name_i,
                     self.data.port_group_name_i, self.extra_specs))
-            self.assertEqual(self.data.inititiatorgroup, ig_from_mv)
+            self.assertEqual(self.data.initiatorgroup_name_f, ig_from_mv)
 
     def test_check_adding_volume_to_storage_group(self):
         with mock.patch.object(
@@ -530,27 +536,88 @@ class PowerMaxMaskingTest(test.TestCase):
                     self.device_id, self.data.masking_view_dict_multiattach)
                 mock_return.assert_called_once()
 
-    @mock.patch.object(rest.PowerMaxRest, 'delete_masking_view')
-    @mock.patch.object(rest.PowerMaxRest, 'delete_initiator_group')
-    @mock.patch.object(rest.PowerMaxRest, 'get_initiator_group')
+    @mock.patch.object(masking.PowerMaxMasking, '_recreate_masking_view')
+    @mock.patch.object(rest.PowerMaxRest, 'get_initiator_group',
+                       return_value=True)
+    def test_verify_initiator_group_from_masking_view(
+            self, mock_get_ig, mock_recreate_mv):
+        mv_dict = deepcopy(self.maskingviewdict)
+        mv_dict['initiator_check'] = True
+        self.mask._verify_initiator_group_from_masking_view(
+            self.data.array, mv_dict['maskingview_name'],
+            mv_dict, self.data.initiatorgroup_name_i,
+            self.data.storagegroup_name_i, self.data.port_group_name_i,
+            self.extra_specs)
+        mock_recreate_mv.assert_called()
+
+    @mock.patch.object(masking.PowerMaxMasking, '_recreate_masking_view')
+    @mock.patch.object(rest.PowerMaxRest, 'get_initiator_group',
+                       return_value=True)
     @mock.patch.object(
         masking.PowerMaxMasking, '_find_initiator_group',
         return_value=tpd.PowerMaxData.initiatorgroup_name_i)
-    def test_verify_initiator_group_from_masking_view(
-            self, mock_find_ig, mock_get_ig, mock_delete_ig, mock_delete_mv):
-        self.mask._verify_initiator_group_from_masking_view(
-            self.data.array, self.maskingviewdict['maskingview_name'],
-            self.maskingviewdict, self.data.initiatorgroup_name_i,
+    def test_verify_initiator_group_from_masking_view_no_recreate(
+            self, mock_find_ig, mock_get_ig, mock_recreate):
+        mv_dict = deepcopy(self.maskingviewdict)
+        mv_dict['initiator_check'] = False
+        self.assertRaises(
+            exception.VolumeBackendAPIException,
+            self.mask._verify_initiator_group_from_masking_view,
+            self.data.array, mv_dict['maskingview_name'],
+            mv_dict, 'OS-Wrong-Host-I-IG',
             self.data.storagegroup_name_i, self.data.port_group_name_i,
             self.extra_specs)
-        mock_get_ig.assert_not_called()
-        mock_get_ig.return_value = False
-        self.mask._verify_initiator_group_from_masking_view(
-            self.data.array, self.maskingviewdict['maskingview_name'],
-            self.maskingviewdict, 'OS-Wrong-Host-I-IG',
+        mock_recreate.assert_not_called()
+
+    @mock.patch.object(rest.PowerMaxRest, 'delete_initiator_group')
+    @mock.patch.object(rest.PowerMaxRest, 'get_initiator_group',
+                       return_value=True)
+    def test_recreate_masking_view(
+            self, mock_get_ig, mock_delete_ig):
+
+        ig_from_conn = self.data.initiatorgroup_name_i
+        ig_from_mv = self.data.initiatorgroup_name_i
+        ig_openstack = self.data.initiatorgroup_name_i
+
+        self.mask._recreate_masking_view(
+            self.data.array, ig_from_conn, ig_from_mv,
+            ig_openstack, self.data.masking_view_name_i, [self.data.initiator],
             self.data.storagegroup_name_i, self.data.port_group_name_i,
             self.extra_specs)
-        mock_get_ig.assert_called()
+        mock_delete_ig.assert_not_called()
+
+    @mock.patch.object(rest.PowerMaxRest, 'delete_initiator_group')
+    @mock.patch.object(rest.PowerMaxRest, 'get_initiator_group',
+                       return_value=True)
+    def test_recreate_masking_view_no_ig_from_connector(
+            self, mock_get_ig, mock_delete_ig):
+
+        ig_from_mv = self.data.initiatorgroup_name_i
+        ig_openstack = self.data.initiatorgroup_name_i
+
+        self.mask._recreate_masking_view(
+            self.data.array, None, ig_from_mv,
+            ig_openstack, self.data.masking_view_name_i, [self.data.initiator],
+            self.data.storagegroup_name_i, self.data.port_group_name_i,
+            self.extra_specs)
+        mock_delete_ig.assert_called()
+
+    @mock.patch.object(rest.PowerMaxRest, 'create_masking_view')
+    @mock.patch.object(rest.PowerMaxRest, 'get_initiator_group',
+                       return_value=True)
+    def test_recreate_masking_view_wrong_host(
+            self, mock_get_ig, mock_create_mv):
+
+        ig_from_conn = 'OS-Wrong-Host-I-IG'
+        ig_from_mv = self.data.initiatorgroup_name_i
+        ig_openstack = self.data.initiatorgroup_name_i
+
+        self.mask._recreate_masking_view(
+            self.data.array, ig_from_conn, ig_from_mv,
+            ig_openstack, self.data.masking_view_name_i, [self.data.initiator],
+            self.data.storagegroup_name_i, self.data.port_group_name_i,
+            self.extra_specs)
+        mock_create_mv.assert_called()
 
     @mock.patch.object(rest.PowerMaxRest, 'delete_masking_view')
     @mock.patch.object(rest.PowerMaxRest, 'delete_initiator_group')
@@ -559,23 +626,19 @@ class PowerMaxMaskingTest(test.TestCase):
     @mock.patch.object(
         masking.PowerMaxMasking, '_find_initiator_group',
         return_value=tpd.PowerMaxData.initiatorgroup_name_i)
-    def test_verify_initiator_group_from_masking_view2(
+    def test_recreate_masking_view_delete_mv(
             self, mock_find_ig, mock_get_ig, mock_delete_ig, mock_delete_mv):
+
         mock_delete_mv.side_effect = [None, Exception]
-        self.mask._verify_initiator_group_from_masking_view(
-            self.data.array, self.maskingviewdict['maskingview_name'],
-            self.maskingviewdict, 'OS-Wrong-Host-I-IG',
+        mv_dict = deepcopy(self.maskingviewdict)
+        mv_dict['initiator_check'] = True
+        verify_flag = self.mask._verify_initiator_group_from_masking_view(
+            self.data.array, mv_dict['maskingview_name'],
+            mv_dict, 'OS-Wrong-Host-I-IG',
             self.data.storagegroup_name_i, self.data.port_group_name_i,
             self.extra_specs)
         mock_delete_mv.assert_called()
-        _, found_ig_from_connector = (
-            self.mask._verify_initiator_group_from_masking_view(
-                self.data.array, self.maskingviewdict['maskingview_name'],
-                self.maskingviewdict, 'OS-Wrong-Host-I-IG',
-                self.data.storagegroup_name_i, self.data.port_group_name_i,
-                self.extra_specs))
-        self.assertEqual(self.data.initiatorgroup_name_i,
-                         found_ig_from_connector)
+        self.assertTrue(verify_flag)
 
     @mock.patch.object(rest.PowerMaxRest, 'create_initiator_group')
     def test_create_initiator_group(self, mock_create_ig):
@@ -1230,3 +1293,58 @@ class PowerMaxMaskingTest(test.TestCase):
             self.data.parent_sg_i)
         mock_create.assert_not_called
         mock_add.assert_not_called()
+
+    @mock.patch.object(rest.PowerMaxRest, 'get_port',
+                       return_value=tpd.PowerMaxData.port_info)
+    @mock.patch.object(rest.PowerMaxRest, 'get_port_ids',
+                       return_value=['FA-1D:4'])
+    def test_check_director_and_port_status(self, mock_port_ids, mock_port):
+        self.mask._check_director_and_port_status(
+            self.data.array, self.data.port_group_name_f)
+
+    @mock.patch.object(rest.PowerMaxRest, 'get_port',
+                       return_value=tpd.PowerMaxData.port_info_off)
+    @mock.patch.object(rest.PowerMaxRest, 'get_port_ids',
+                       return_value=['FA-1D:4'])
+    def test_check_director_and_port_status_invalid_status(
+            self, mock_port_ids, mock_port):
+        exception_message = (
+            r"The director status is Offline and the port status is OFF for "
+            r"dir:port FA-1D:4.")
+
+        with self.assertRaisesRegex(
+                exception.VolumeBackendAPIException,
+                exception_message):
+            self.mask._check_director_and_port_status(
+                self.data.array, self.data.port_group_name_f)
+
+    @mock.patch.object(rest.PowerMaxRest, 'get_port',
+                       return_value=tpd.PowerMaxData.port_info_no_status)
+    @mock.patch.object(rest.PowerMaxRest, 'get_port_ids',
+                       return_value=['FA-1D:4'])
+    def test_check_director_and_port_status_no_status(
+            self, mock_port_ids, mock_port):
+        exception_message = (
+            r"Unable to get the director or port status for dir:port "
+            r"FA-1D:4.")
+
+        with self.assertRaisesRegex(
+                exception.VolumeBackendAPIException,
+                exception_message):
+            self.mask._check_director_and_port_status(
+                self.data.array, self.data.port_group_name_f)
+
+    @mock.patch.object(rest.PowerMaxRest, 'get_port',
+                       return_value=tpd.PowerMaxData.port_info_no_details)
+    @mock.patch.object(rest.PowerMaxRest, 'get_port_ids',
+                       return_value=['FA-1D:4'])
+    def test_check_director_and_port_status_no_details(
+            self, mock_port_ids, mock_port):
+        exception_message = (
+            r"Unable to get port information for dir:port FA-1D:4.")
+
+        with self.assertRaisesRegex(
+                exception.VolumeBackendAPIException,
+                exception_message):
+            self.mask._check_director_and_port_status(
+                self.data.array, self.data.port_group_name_f)
