@@ -1,4 +1,4 @@
-# Copyright (c) 2017-2019 Dell Inc. or its subsidiaries.
+# Copyright (c) 2020 Dell Inc. or its subsidiaries.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -23,7 +23,7 @@ import six
 
 from cinder import exception
 from cinder.objects import fields
-from cinder import test
+from cinder.tests.unit import test
 from cinder.tests.unit.volume.drivers.dell_emc.powermax import (
     powermax_data as tpd)
 from cinder.tests.unit.volume.drivers.dell_emc.powermax import (
@@ -44,8 +44,9 @@ class PowerMaxUtilsTest(test.TestCase):
         self.replication_device = self.data.sync_rep_device
         configuration = tpfo.FakeConfiguration(
             None, 'UtilsTests', 1, 1, san_ip='1.1.1.1', san_login='smc',
-            vmax_array=self.data.array, vmax_srp='SRP_1', san_password='smc',
-            san_api_port=8443, vmax_port_groups=[self.data.port_group_name_i],
+            powermax_array=self.data.array, powermax_srp='SRP_1',
+            san_password='smc', san_api_port=8443,
+            powermax_port_groups=[self.data.port_group_name_i],
             replication_device=self.replication_device)
         rest.PowerMaxRest._establish_rest_session = mock.Mock(
             return_value=tpfo.FakeRequestsSession())
@@ -219,42 +220,53 @@ class PowerMaxUtilsTest(test.TestCase):
     def test_is_compression_disabled_true(self):
         # Compression disabled in extra specs
         extra_specs = self.data.extra_specs_disable_compression
-        do_disable_compression = self.utils.is_compression_disabled(
-            extra_specs)
-        self.assertTrue(do_disable_compression)
+        self.assertTrue(self.utils.is_compression_disabled(extra_specs))
         # Compression disabled by no SL/WL combination
         extra_specs = deepcopy(self.data.vol_type_extra_specs_none_pool)
-        self.assertTrue(self.utils.is_compression_disabled(
-            extra_specs))
+        self.assertTrue(self.utils.is_compression_disabled(extra_specs))
+        extra_specs3 = deepcopy(extra_specs)
+        extra_specs3.update({utils.DISABLECOMPRESSION: '<is> True'})
+        self.assertTrue(self.utils.is_compression_disabled(extra_specs3))
+        extra_specs4 = deepcopy(extra_specs)
+        extra_specs4.update({utils.DISABLECOMPRESSION: 'True'})
+        self.assertTrue(self.utils.is_compression_disabled(extra_specs4))
 
     def test_is_compression_disabled_false(self):
         # Path 1: no compression extra spec set
         extra_specs = self.data.extra_specs
-        do_disable_compression = self.utils.is_compression_disabled(
-            extra_specs)
-        self.assertFalse(do_disable_compression)
+        self.assertFalse(self.utils.is_compression_disabled(extra_specs))
         # Path 2: compression extra spec set to false
         extra_specs2 = deepcopy(extra_specs)
         extra_specs2.update({utils.DISABLECOMPRESSION: 'false'})
-        do_disable_compression2 = self.utils.is_compression_disabled(
-            extra_specs)
-        self.assertFalse(do_disable_compression2)
+        self.assertFalse(self.utils.is_compression_disabled(extra_specs2))
+        extra_specs3 = deepcopy(extra_specs)
+        extra_specs3.update({utils.DISABLECOMPRESSION: '<is> False'})
+        self.assertFalse(self.utils.is_compression_disabled(extra_specs3))
+        extra_specs4 = deepcopy(extra_specs)
+        extra_specs4.update({utils.DISABLECOMPRESSION: 'False'})
+        self.assertFalse(self.utils.is_compression_disabled(extra_specs4))
 
     def test_change_compression_type_true(self):
         source_compr_disabled = True
-        new_type_compr_disabled = {
+        new_type_compr_disabled_1 = {
             'extra_specs': {utils.DISABLECOMPRESSION: 'false'}}
-        ans = self.utils.change_compression_type(
-            source_compr_disabled, new_type_compr_disabled)
-        self.assertTrue(ans)
+        self.assertTrue(self.utils.change_compression_type(
+            source_compr_disabled, new_type_compr_disabled_1))
+        new_type_compr_disabled_2 = {
+            'extra_specs': {utils.DISABLECOMPRESSION: '<is> False'}}
+        self.assertTrue(self.utils.change_compression_type(
+            source_compr_disabled, new_type_compr_disabled_2))
 
     def test_change_compression_type_false(self):
         source_compr_disabled = True
         new_type_compr_disabled = {
             'extra_specs': {utils.DISABLECOMPRESSION: 'true'}}
-        ans = self.utils.change_compression_type(
-            source_compr_disabled, new_type_compr_disabled)
-        self.assertFalse(ans)
+        self.assertFalse(self.utils.change_compression_type(
+            source_compr_disabled, new_type_compr_disabled))
+        new_type_compr_disabled_2 = {
+            'extra_specs': {utils.DISABLECOMPRESSION: '<is> True'}}
+        self.assertFalse(self.utils.change_compression_type(
+            source_compr_disabled, new_type_compr_disabled_2))
 
     def test_is_replication_enabled(self):
         is_re = self.utils.is_replication_enabled(
@@ -354,6 +366,23 @@ class PowerMaxUtilsTest(test.TestCase):
                      {'pool_name': 'Diamond+SRP_1+000197800111'}]
         self.assertEqual(ref_pools, new_pools)
 
+    def test_add_promotion_pools(self):
+        array = self.data.array
+        pools = [{'pool_name': 'Diamond+None+SRP_1+000197800111',
+                  'location_info': '000197800111#SRP_1#None#Diamond'},
+                 {'pool_name': 'Gold+OLTP+SRP_1+000197800111',
+                  'location_info': '000197800111#SRP_1#OLTP#Gold'}]
+        new_pools = self.utils.add_promotion_pools(pools, array)
+        ref_pools = [{'pool_name': 'Diamond+None+SRP_1+000197800111',
+                      'location_info': '000197800111#SRP_1#None#Diamond'},
+                     {'pool_name': 'Gold+OLTP+SRP_1+000197800111',
+                      'location_info': '000197800111#SRP_1#OLTP#Gold'},
+                     {'pool_name': 'Diamond+None+SRP_1+000197800123',
+                      'location_info': '000197800123#SRP_1#None#Diamond'},
+                     {'pool_name': 'Gold+OLTP+SRP_1+000197800123',
+                      'location_info': '000197800123#SRP_1#OLTP#Gold'}]
+        self.assertEqual(ref_pools, new_pools)
+
     def test_update_volume_group_name(self):
         group = self.data.test_group_1
         ref_group_name = self.data.test_vol_grp_name
@@ -1211,7 +1240,7 @@ class PowerMaxUtilsTest(test.TestCase):
             'array': self.data.array,
             'editStorageGroupActionParam': {
                 'expandStorageGroupParam': {
-                    'addVolumeParam': {}}}}
+                    'addVolumeParam': {'create_new_volumes': 'False'}}}}
 
         updated_payload = self.utils.update_payload_for_rdf_vol_create(
             payload, self.data.remote_array, self.data.storagegroup_name_f)
@@ -1220,6 +1249,7 @@ class PowerMaxUtilsTest(test.TestCase):
             'editStorageGroupActionParam': {
                 'expandStorageGroupParam': {
                     'addVolumeParam': {
+                        'create_new_volumes': 'True',
                         'remoteSymmSGInfoParam': {
                             'force': 'true',
                             'remote_symmetrix_1_id': self.data.remote_array,
@@ -1277,6 +1307,15 @@ class PowerMaxUtilsTest(test.TestCase):
             self.utils.validate_multiple_rep_device,
             rep_devices)
 
+    def test_validate_multiple_rep_device_promotion_start_backend_id(self):
+        backend_id = utils.PMAX_FAILOVER_START_ARRAY_PROMOTION
+        rep_devices = deepcopy(self.data.multi_rep_device)
+        rep_devices[0][utils.BACKEND_ID] = backend_id
+        self.assertRaises(
+            exception.InvalidConfigurationValue,
+            self.utils.validate_multiple_rep_device,
+            rep_devices)
+
     def test_validate_multiple_rep_device_missing_backend_id(self):
         rep_devices = deepcopy(self.data.multi_rep_device)
         rep_devices[0].pop(utils.BACKEND_ID)
@@ -1357,6 +1396,12 @@ class PowerMaxUtilsTest(test.TestCase):
             excep_msg = str(e)
             self.assertIn(expected_str, excep_msg)
 
+    def test_get_rep_config_promotion_stats(self):
+        rep_configs = self.data.multi_rep_config_list
+        backend_id = 'testing'
+        rep_device = self.utils.get_rep_config(backend_id, rep_configs, True)
+        self.assertEqual(rep_configs[0], rep_device)
+
     def test_get_replication_targets(self):
         rep_targets_expected = [self.data.remote_array]
         rep_configs = self.data.multi_rep_config_list
@@ -1365,31 +1410,77 @@ class PowerMaxUtilsTest(test.TestCase):
 
     def test_validate_failover_request_success(self):
         is_failed_over = False
+        is_promoted = False
         failover_backend_id = self.data.rep_backend_id_sync
         rep_configs = self.data.multi_rep_config_list
+        primary_array = self.data.array
+        array_list = [self.data.array]
         is_valid, msg = self.utils.validate_failover_request(
-            is_failed_over, failover_backend_id, rep_configs)
+            is_failed_over, failover_backend_id, rep_configs,
+            primary_array, array_list, is_promoted)
         self.assertTrue(is_valid)
         self.assertEqual("", msg)
 
     def test_validate_failover_request_already_failed_over(self):
         is_failed_over = True
+        is_promoted = False
         failover_backend_id = self.data.rep_backend_id_sync
         rep_configs = self.data.multi_rep_config_list
+        primary_array = self.data.array
+        array_list = [self.data.array]
         is_valid, msg = self.utils.validate_failover_request(
-            is_failed_over, failover_backend_id, rep_configs)
+            is_failed_over, failover_backend_id, rep_configs,
+            primary_array, array_list, is_promoted)
         self.assertFalse(is_valid)
         expected_msg = ('Cannot failover, the backend is already in a failed '
                         'over state, if you meant to failback, please add '
                         '--backend_id default to the command.')
         self.assertEqual(expected_msg, msg)
 
-    def test_validate_failover_request_invalid_failback(self):
-        is_failed_over = False
+    def test_validate_failover_request_failback_missing_array(self):
+        is_failed_over = True
+        is_promoted = False
         failover_backend_id = 'default'
         rep_configs = self.data.multi_rep_config_list
+        primary_array = self.data.array
+        array_list = [self.data.remote_array]
         is_valid, msg = self.utils.validate_failover_request(
-            is_failed_over, failover_backend_id, rep_configs)
+            is_failed_over, failover_backend_id, rep_configs,
+            primary_array, array_list, is_promoted)
+        self.assertFalse(is_valid)
+        expected_msg = ('Cannot failback, the configured primary array is '
+                        'not currently available to perform failback to. '
+                        'Please ensure array %s is visible in '
+                        'Unisphere.') % primary_array
+        self.assertEqual(expected_msg, msg)
+
+    def test_validate_failover_request_promotion_finalize(self):
+        is_failed_over = True
+        is_promoted = True
+        failover_backend_id = utils.PMAX_FAILOVER_START_ARRAY_PROMOTION
+        rep_configs = self.data.multi_rep_config_list
+        primary_array = self.data.array
+        array_list = [self.data.array]
+        is_valid, msg = self.utils.validate_failover_request(
+            is_failed_over, failover_backend_id, rep_configs,
+            primary_array, array_list, is_promoted)
+        self.assertFalse(is_valid)
+        expected_msg = ('Failover promotion currently in progress, please '
+                        'finish the promotion process and issue a failover '
+                        'using the "default" backend_id to complete this '
+                        'process.')
+        self.assertEqual(expected_msg, msg)
+
+    def test_validate_failover_request_invalid_failback(self):
+        is_failed_over = False
+        is_promoted = False
+        failover_backend_id = 'default'
+        rep_configs = self.data.multi_rep_config_list
+        primary_array = self.data.array
+        array_list = [self.data.array]
+        is_valid, msg = self.utils.validate_failover_request(
+            is_failed_over, failover_backend_id, rep_configs,
+            primary_array, array_list, is_promoted)
         self.assertFalse(is_valid)
         expected_msg = ('Cannot failback, backend is not in a failed over '
                         'state. If you meant to failover, please either omit '
@@ -1397,28 +1488,6 @@ class PowerMaxUtilsTest(test.TestCase):
                         'parameter with a valid backend id.')
         self.assertEqual(expected_msg, msg)
 
-    def test_validate_failover_request_no_backend_id_multi_rep(self):
-        is_failed_over = False
-        failover_backend_id = None
-        rep_configs = self.data.multi_rep_config_list
-        is_valid, msg = self.utils.validate_failover_request(
-            is_failed_over, failover_backend_id, rep_configs)
-        self.assertFalse(is_valid)
-        expected_msg = ('Cannot failover, no backend_id provided while '
-                        'multiple replication devices are defined in '
-                        'cinder.conf, please provide a backend_id '
-                        'which will act as new primary array by '
-                        'appending --backend_id <id> to your command.')
-        self.assertEqual(expected_msg, msg)
-
-    def test_validate_failover_request_incorrect_backend_id_multi_rep(self):
-        is_failed_over = False
-        failover_backend_id = 'invalid_id'
-        rep_configs = self.data.multi_rep_config_list
-        self.assertRaises(exception.InvalidInput,
-                          self.utils.validate_failover_request,
-                          is_failed_over, failover_backend_id, rep_configs)
-
     def test_validate_replication_group_config_success(self):
         rep_configs = deepcopy(self.data.multi_rep_config_list)
         extra_specs = deepcopy(
