@@ -135,14 +135,14 @@ class PowerMaxISCSIDriver(san.SanISCSIDriver):
               - Fix to enable legacy volumes to live migrate (#1867163)
               - Use of snap id instead of generation (bp powermax-snapset-ids)
               - Support for Failover Abilities (bp/powermax-failover-abilities)
-        4.3.1 - Fix non-temporary snapshot delete (#1887962)
-        4.3.2 - Fix for create snapshot (#1939139)
+        4.4.0 - Early check for status of port
+        4.4.1 - Allow for case mismatch in SGs (bug #1929429)
     """
 
-    VERSION = "4.3.2"
+    VERSION = "4.4.1"
 
     # ThirdPartySystems wiki
-    CI_WIKI_NAME = "EMC_VMAX_CI"
+    CI_WIKI_NAME = "DellEMC_PowerMAX_CI"
 
     def __init__(self, *args, **kwargs):
 
