@@ -22,7 +22,6 @@ from oslo_utils import excutils
 
 from cinder import coordination
 from cinder import exception
-from cinder import utils as cinder_utils
 from cinder.volume import configuration
 from cinder.volume.drivers.hitachi import hbsd_utils as utils
 from cinder.volume import volume_utils
@@ -138,8 +137,6 @@ class HBSDCommon():
             'portals': {},
         }
 
-        self._stats = {}
-
     def create_ldev(self, size):
         """Create an LDEV and return its LDEV number."""
         raise NotImplementedError()
@@ -332,11 +329,10 @@ class HBSDCommon():
                 backend_state='down'))
             data["pools"].append(single_pool)
             LOG.debug("Updating volume status. (%s)", data)
-            self._stats = data
             utils.output_log(
                 MSG.POOL_INFO_RETRIEVAL_FAILED,
                 pool=self.conf.hitachi_pool)
-            return
+            return data
         single_pool.update(dict(
             total_capacity_gb=total_capacity,
             free_capacity_gb=free_capacity,
@@ -350,13 +346,7 @@ class HBSDCommon():
         single_pool.update(dict(backend_state='up'))
         data["pools"].append(single_pool)
         LOG.debug("Updating volume status. (%s)", data)
-        self._stats = data
-
-    def get_volume_stats(self, refresh=False):
-        """Return properties, capabilities and current states of the driver."""
-        if refresh:
-            self.update_volume_stats()
-        return self._stats
+        return data
 
     def discard_zero_page(self, volume):
         """Return the volume's no-data pages to the storage pool."""
@@ -582,7 +572,7 @@ class HBSDCommon():
         """Initialize server-storage connection."""
         targets = kwargs.pop(
             'targets', {'info': {}, 'list': [], 'iqns': {}, 'target_map': {}})
-        connector = cinder_utils.brick_get_connector_properties(
+        connector = volume_utils.brick_get_connector_properties(
             multipath=self.conf.use_multipath_for_image_xfer,
             enforce_multipath=self.conf.enforce_multipath_for_image_xfer)
         target_ports = self.storage_info['controller_ports']
