@@ -302,7 +302,7 @@ class RBDDriver(driver.CloneableImageVD, driver.MigrateVD,
     def _show_msg_check_clone_v2_api(self, volume_name):
         if not self._clone_v2_api_checked:
             self._clone_v2_api_checked = True
-            with RBDVolumeProxy(self, volume_name) as volume:
+            with RBDVolumeProxy(self, volume_name, read_only=True) as volume:
                 try:
                     if (volume.volume.op_features() &
                             self.rbd.RBD_OPERATION_FEATURE_CLONE_PARENT):
@@ -669,7 +669,9 @@ class RBDDriver(driver.CloneableImageVD, driver.MigrateVD,
 
     def _get_clone_depth(self, client, volume_name, depth=0):
         """Returns the number of ancestral clones of the given volume."""
-        parent_volume = self.rbd.Image(client.ioctx, volume_name)
+        parent_volume = self.rbd.Image(client.ioctx,
+                                       volume_name,
+                                       read_only=True)
         try:
             _pool, parent, _snap = self._get_clone_info(parent_volume,
                                                         volume_name)
@@ -1016,7 +1018,7 @@ class RBDDriver(driver.CloneableImageVD, driver.MigrateVD,
         default_stripe_unit = \
             self.configuration.rbd_store_chunk_size * units.Mi
 
-        image = self.rbd.Image(ioctx, volume_name)
+        image = self.rbd.Image(ioctx, volume_name, read_only=True)
         try:
             image_stripe_unit = image.stripe_unit()
         finally:
@@ -1211,8 +1213,6 @@ class RBDDriver(driver.CloneableImageVD, driver.MigrateVD,
                         clone_snap = snap['name']
                         break
 
-                    raise exception.VolumeIsBusy(volume_name=volume_name)
-
                 # Determine if this volume is itself a clone
                 _pool, parent, parent_snap = self._get_clone_info(rbd_image,
                                                                   volume_name,
@@ -1225,13 +1225,23 @@ class RBDDriver(driver.CloneableImageVD, driver.MigrateVD,
                          self.configuration.rados_connection_retries)
             def _try_remove_volume(client, volume_name):
                 if self.configuration.enable_deferred_deletion:
-                    LOG.debug("moving volume %s to trash", volume_name)
                     delay = self.configuration.deferred_deletion_delay
-                    self.RBDProxy().trash_move(client.ioctx,
-                                               volume_name,
-                                               delay)
                 else:
-                    self.RBDProxy().remove(client.ioctx, volume_name)
+                    try:
+                        self.RBDProxy().remove(client.ioctx, volume_name)
+                        return
+                    except (self.rbd.ImageHasSnapshots, self.rbd.ImageBusy):
+                        delay = 0
+                LOG.debug("moving volume %s to trash", volume_name)
+                # When using the RBD v2 clone api, deleting a volume
+                # that has a snapshot in the trash space raises a
+                # busy exception.
+                # In order to solve this, call the trash operation
+                # which should succeed when the volume has
+                # dependencies.
+                self.RBDProxy().trash_move(client.ioctx,
+                                           volume_name,
+                                           delay)
 
             if clone_snap is None:
                 LOG.debug("deleting rbd volume %s", volume_name)
@@ -1767,7 +1777,9 @@ class RBDDriver(driver.CloneableImageVD, driver.MigrateVD,
         with RADOSClient(self) as client:
             # Raise an exception if we didn't find a suitable rbd image.
             try:
-                rbd_image = self.rbd.Image(client.ioctx, rbd_name)
+                rbd_image = self.rbd.Image(client.ioctx,
+                                           rbd_name,
+                                           read_only=True)
             except self.rbd.ImageNotFound:
                 kwargs = {'existing_ref': rbd_name,
                           'reason': 'Specified rbd image does not exist.'}
@@ -1994,7 +2006,8 @@ class RBDDriver(driver.CloneableImageVD, driver.MigrateVD,
             # Raise an exception if we didn't find a suitable rbd image.
             try:
                 rbd_snapshot = self.rbd.Image(client.ioctx, volume_name,
-                                              snapshot=snapshot_name)
+                                              snapshot=snapshot_name,
+                                              read_only=True)
             except self.rbd.ImageNotFound:
                 kwargs = {'existing_ref': snapshot_name,
                           'reason': 'Specified snapshot does not exist.'}
