@@ -325,13 +325,12 @@ class RemoteFSDriver(driver.BaseVD):
             self._create_regular_file(volume_path, volume_size)
 
         self._set_rw_permissions(volume_path)
-        if not volume.consistencygroup_id and not volume.group_id:
-            volume.admin_metadata['format'] = self.format
-            # This is done here because when creating a volume from image,
-            # while encountering other volume.save() method fails for
-            # non-admins
-            with volume.obj_as_admin():
-                volume.save()
+        volume.admin_metadata['format'] = self.format
+        # This is done here because when creating a volume from image,
+        # while encountering other volume.save() method fails for
+        # non-admins
+        with volume.obj_as_admin():
+            volume.save()
 
     def _ensure_shares_mounted(self):
         """Look for remote shares in the flags and mount them locally."""
@@ -1394,7 +1393,7 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
                   {'vol': volume.id, 'snap': snapshot.id})
 
         status = snapshot.status
-        acceptable_states = ['available', 'backing-up']
+        acceptable_states = ['available', 'backing-up', 'restoring']
         self._validate_state(status, acceptable_states,
                              obj_description='snapshot',
                              invalid_exc=exception.InvalidSnapshot)
